#!/bin/sh
# find out where the build script is located
tdir=`echo "$0" | sed 's%[\\/][^\\/][^\\/]*$%%'`
test "x$tdir" = "x$0" && tdir=.
scriptdir=`cd $tdir; pwd`
# find src directory.  
# If scriptdir ends in SPECS, then topdir is $scriptdir/.. 
# If scriptdir ends in CYGWIN-PATCHES, then topdir is $scriptdir/../..
# Otherwise, we assume that topdir = scriptdir
topdir1=`echo ${scriptdir} | sed 's%/SPECS$%%'`
topdir2=`echo ${scriptdir} | sed 's%/CYGWIN-PATCHES$%%'`
if [ "x$topdir1" != "x$scriptdir" ] ; then # SPECS
  topdir=`cd ${scriptdir}/..; pwd`
else
  if [ "x$topdir2" != "x$scriptdir" ] ; then # CYGWIN-PATCHES
    topdir=`cd ${scriptdir}/../..; pwd`
  else
    topdir=`cd ${scriptdir}; pwd`
  fi
fi

tscriptname=`basename $0 .sh`
export PKG=`echo $tscriptname | sed -e 's/\-[^\-]*\-[^\-]*$//'`
export VER=`echo $tscriptname | sed -e "s/${PKG}\-//" -e 's/\-[^\-]*$//'`
export REL=`echo $tscriptname | sed -e "s/${PKG}\-${VER}\-//"`
export FULLPKG=${PKG}-${VER}-${REL}

# determine correct decompression option and tarball filename
if [ -e ${PKG}-${VER}.tar.gz ] ; then
  export opt_decomp=z
  export src_orig_pkg_ext=gz
elif [ -e ${PKG}-${VER}.tar.bz2 ] ; then
  export opt_decomp=j
  export src_orig_pkg_ext=bz2
else
  echo Cannot find original package.
  exit 1
fi

export src_orig_pkg_name=${PKG}-${VER}.tar.${src_orig_pkg_ext}
export src_pkg_name=${FULLPKG}-src.tar.bz2
export src_patch_name=${FULLPKG}.patch
export bin_pkg_name=${FULLPKG}.tar.bz2

export src_orig_pkg=${topdir}/${src_orig_pkg_name}
export src_pkg=${topdir}/${src_pkg_name}
export src_patch=${topdir}/${src_patch_name}
export bin_pkg=${topdir}/${bin_pkg_name}
export srcdir=${topdir}/${PKG}-${VER}
export objdir=${srcdir}/.build
export instdir=${srcdir}/.inst
export srcinstdir=${srcdir}/.sinst
export configurelogfile=${topdir}/${FULLPKG}-CONFIGURE.LOG
export makelogfile=${topdir}/${FULLPKG}-MAKE.LOG
export checklogfile=${topdir}/${FULLPKG}-CHECK.LOG
export installlogfile=${topdir}/${FULLPKG}-INSTALL.LOG

# run on
host=i686-pc-cygwin
# if this package creates binaries, they run on
target=i686-pc-cygwin
prefix=/usr
sysconfdir=/etc
MY_CFLAGS="-O2"
MY_LDFLAGS=

mkdirs() {
  (cd ${topdir} && \
  rm -fr ${objdir} ${instdir} ${srcinstdir} && \
  mkdir -p ${objdir} && \
  mkdir -p ${instdir} && \
  mkdir -p ${srcinstdir} )
}
prep() {
  (cd ${topdir} && \
  tar xv${opt_decomp}f ${src_orig_pkg} ; \
  cd ${topdir} && \
  patch -p0 < ${src_patch} 
  && mkdirs )
}
manpatch() {
  cd ${srcdir} && \
  patch -p1 < CYGWIN-PATCHES/man-1.5m2-apropos.patch
#  patch -p1 < CYGWIN-PATCHES/man-1.5m2-lc_all.patch
#  patch -p1 < CYGWIN-PATCHES/man-1.5o1-i18n_nroff.patch
#  patch -p1 < CYGWIN-PATCHES/man-1.5m2-i18n_makewhatis.patch
#  patch -p1 < CYGWIN-PATCHES/man-1.5p-makewhatis2.patch
#  patch -p1 < CYGWIN-PATCHES/man-1.5p-makewhatis3.patch
#  patch -p1 < CYGWIN-PATCHES/man-1.5o1-man.1.patch
#  patch -p1 < CYGWIN-PATCHES/man-1.5o1-nocolor.patch
#  patch -p1 < CYGWIN-PATCHES/man-1.5o1-manwidth.patch
#  patch -p1 < CYGWIN-PATCHES/man-1.5o1-i18n_iconv.patch
  patch -p1 < CYGWIN-PATCHES/man-1.5p-makewhatis.patch
  patch -p1 < CYGWIN-PATCHES/man-1.5p-cygwin-de-mesgs.patch
  patch -p1 < CYGWIN-PATCHES/man-1.5p-cygwin-gencat-destdir-exeext.patch
  patch -p1 < CYGWIN-PATCHES/man-1.5p-cygwin-man-config.patch
  patch -p1 < CYGWIN-PATCHES/man-1.5p-cygwin-cygwin32.patch
  patch -p1 < CYGWIN-PATCHES/man-1.5p-cygwin-safe.patch
  patch -p1 < CYGWIN-PATCHES/man-1.5p-cygwin-safe2.patch
  patch -p1 < CYGWIN-PATCHES/man-1.5p-cygwin-man-sections.patch
  patch -p1 < CYGWIN-PATCHES/man-1.5p-cygwin-minus-o-s-flag.patch
  patch -p1 < CYGWIN-PATCHES/man-1.5p-cygwin-makewhatis.patch
  patch -p1 < CYGWIN-PATCHES/man-1.5p-cygwin-configure.patch
  patch -p1 < CYGWIN-PATCHES/man-1.5p-cygwin-termios.patch
  patch -p1 < CYGWIN-PATCHES/man-1.5p-cygwin-configure2.patch
  patch -p1 < CYGWIN-PATCHES/man-1.5p-cygwin-man-conf.patch
#  patch -p1 < CYGWIN-PATCHES/man-1.5p-cygwin-specific.patch
#  patch -p1 < CYGWIN-PATCHES/man-1.5p-mandirlist.patch
#  patch -p1 < CYGWIN-PATCHES/man-1.5p-man2html.patch
}
conf() {
  (cd ${objdir}                                 && \
  lndir ../                                     && \
  rm -rf .build .inst .sinst CYGWIN-PATCHES     && \
  CFLAGS="${MY_CFLAGS}" LDFLAGS="${MY_LDFLAGS}"    \
  CC="gcc -O2 -DNONLS -DLC_MESSAGES=6"             \
  ${srcdir}/configure +fhs +lang all               \
  2>&1                                             \
  | tee ${configurelogfile} )
}
build() {
  (cd ${objdir} && \
  CFLAGS="${MY_CFLAGS}" make EXEEXT=.exe \
  2>&1 | tee ${makelogfile} )
}
check() {
  (cd ${objdir} && \
  make test 2>&1 | tee ${checklogfile} )
}
clean() {
  (cd ${objdir} && \
  make clean )
}
install() {
  (cd ${objdir} && \
  make install DESTDIR=${instdir}   \
  2>&1                              \
  | tee -a ${installlogfile}     && \
  for f in ${prefix}/share/info/dir ${prefix}/info/dir ; do
    if [ -f ${instdir}${f} ] ; then \
      rm -f ${instdir}${f} ; \
    fi ;\
  done &&\
  for d in ${prefix}/share/doc/${PKG}-${VER} ${prefix}/share/doc/Cygwin \
           ${sysconfdir}/postinstall ${sysconfdir}/preremove ; do
    if [ ! -d ${instdir}${d} ] ; then
      mkdir -p ${instdir}${d} ;\
    fi ;\
  done &&\
  templist=""; \
  for f in ${srcdir}/ANNOUNCE ${srcdir}/CHANGES ${srcdir}/INSTALL \
           ${srcdir}/KNOWNBUG ${srcdir}/LICENSE ${srcdir}/README* \
           ${srcdir}/TODO ${srcdir}/COPYING ${srcdir}/HISTORY ; do \
    if [ -f $f ] ; then \
      templist="$templist $f"; \
    fi ; \
  done && \
  if [ ! "x$templist" = "x" ]; then \
    /usr/bin/install -m 644 $templist \
         ${instdir}${prefix}/share/doc/${PKG}-${VER} ;
  fi && \
  if [ -f ${srcdir}/CYGWIN-PATCHES/${PKG}.README ]; then \
    /usr/bin/install -m 644 ${srcdir}/CYGWIN-PATCHES/${PKG}.README \
      ${instdir}${prefix}/share/doc/Cygwin/${PKG}-${VER}.README ; \
  else \
    if [ -f ${srcdir}/CYGWIN-PATCHES/README ]; then \
      /usr/bin/install -m 644 ${srcdir}/CYGWIN-PATCHES/README \
        ${instdir}${prefix}/share/doc/Cygwin/${PKG}-${VER}.README ; \
    fi ;\
  fi ;\
  if [ -f ${srcdir}/CYGWIN-PATCHES/postinstall.sh ] ; then \
  /usr/bin/install -m 755 ${srcdir}/CYGWIN-PATCHES/postinstall.sh \
      ${instdir}${sysconfdir}/postinstall/${PKG}.sh
  fi && \
  if [ -f ${srcdir}/CYGWIN-PATCHES/preremove.sh ] ; then \
  /usr/bin/install -m 755 ${srcdir}/CYGWIN-PATCHES/preremove.sh \
      ${instdir}${sysconfdir}/preremove/${PKG}.sh
  fi && \
  if [ -f ${srcdir}/CYGWIN-PATCHES/manifest.lst ] ; then \
  /usr/bin/install -m 644 ${srcdir}/CYGWIN-PATCHES/manifest.lst \
      ${instdir}${sysconfdir}/preremove/${PKG}-manifest.lst
  fi && \
  /usr/bin/install -m 644 ${srcdir}/misc/manlint.1                      \
                          ${instdir}/usr/share/man/man1              && \
  /usr/bin/install -m 644 ${srcdir}/misc/manlint                        \
                          ${instdir}/usr/bin                         && \
  cd ${instdir}/usr/bin                                              && \
  ln -sf man.exe manpath.exe                                         && \
  cd ${instdir}/usr/share/man/man1                                   && \
  ln -sf man.1 manpath.1                                             && \
  rm -rf ${instdir}${prefix}/share/locale                            && \
  rm -rf ${instdir}${prefix}/share/man/bg/man8                       && \
  rm -rf ${instdir}${prefix}/share/man/cs/man8                       && \
  rm -rf ${instdir}${prefix}/share/man/da/man8                       && \
  rm -rf ${instdir}${prefix}/share/man/de/man8                       && \
  rm -rf ${instdir}${prefix}/share/man/es/man8                       && \
  rm -rf ${instdir}${prefix}/share/man/fi/man8                       && \
  rm -rf ${instdir}${prefix}/share/man/fr/man8                       && \
  rm -rf ${instdir}${prefix}/share/man/hr/man8                       && \
  rm -rf ${instdir}${prefix}/share/man/ja/man8                       && \
  rm -rf ${instdir}${prefix}/share/man/ko/man8                       && \
  rm -rf ${instdir}${prefix}/share/man/nl/man8                       && \
  rm -rf ${instdir}${prefix}/share/man/pl/man8                       && \
  rm -rf ${instdir}${prefix}/share/man/pt/man8                       && \
  rm -rf ${instdir}${prefix}/share/man/sl/man8                       && \
  mkdir -p ${instdir}/var/cache/man                                  && \
  mkdir -p ${instdir}/etc/defaults/usr/share/misc                    && \
  mv ${instdir}/usr/share/misc/man.conf                                 \
     ${instdir}/etc/defaults/usr/share/misc/man.conf )
}
strip() {
  (cd ${instdir} && \
  find . -name "*.exe" -or -name "*.dll" | xargs strip > /dev/null 2>&1
  true )
}
list() {
  (cd ${instdir} && \
  find . -name "*" ! -type d | sed 's%^\.%  %' ; \
  true )
}
listdep() {
  (cd ${instdir} && \
   find . -name '*.exe' -o -name '*.dll' | xargs cygcheck | \
     sed -e '/\.exe/d' -e 's,\\,/,g' | sort -bu | xargs -n1 cygpath -u | \
     xargs cygcheck -f | uniq
   true )
}
pkg() {
  (cd ${instdir} && \
  tar cvjf ${bin_pkg} * )
}
mkpatch() {
  (cd ${srcdir} && \
  find . -name "autom4te.cache" | xargs rm -rf > /dev/null
  tar xv${opt_decomp}f ${src_orig_pkg} ;\
  mv ${PKG}-${VER} ../${PKG}-${VER}-orig && \
  cd ${topdir} && \
  diff -urN -x '.build' -x '.inst' -x '.sinst' \
    ${PKG}-${VER}-orig ${PKG}-${VER} > \
    ${srcinstdir}/${src_patch_name} ; \
  rm -rf ${PKG}-${VER}-orig )
}
spkg() {
  (cd ${topdir} && \
  cp ${src_orig_pkg} ${srcinstdir}/${src_orig_pkg_name} && \
  cp $0 ${srcinstdir}/`basename $0` && \
  cd ${srcinstdir} && \
  tar cvjf ${src_pkg} * )
}
finish() {
  rm -rf ${srcdir}
  rm ${configurelogfile} > /dev/null 2>&1
  rm ${makelogfile}      > /dev/null 2>&1
  rm ${checklogfile}     > /dev/null 2>&1
  rm ${installlogfile}   > /dev/null 2>&1 
}
case $1 in
  prep)	          prep ; STATUS=$? ;;
  mkdirs)	mkdirs ; STATUS=$? ;;
  conf)	          conf ; STATUS=$? ;;
  build)	 build ; STATUS=$? ;;
  check)	 check ; STATUS=$? ;;
  clean)	 clean ; STATUS=$? ;;
  install)     install ; STATUS=$? ;;
  list)	          list ; STATUS=$? ;;
  listdep)     listdep ; STATUS=$? ;;
  strip)	 strip ; STATUS=$? ;;
  package)	   pkg ; STATUS=$? ;;
  pkg)	           pkg ; STATUS=$? ;;
  mkpatch)     mkpatch ; STATUS=$? ;;
  manpatch)   manpatch ; STATUS=$? ;;
  src-package)	  spkg ; STATUS=$? ;;
  spkg)	          spkg ; STATUS=$? ;;
  finish)       finish ; STATUS=$? ;;
  all) prep && mkpatch && manpatch && conf && build && install && \
      strip &&  pkg &&  spkg && finish ; \
	  STATUS=$? ;;
  *) echo "Error: bad arguments" ; exit 1 ;;
esac
exit ${STATUS}
